﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Core;

namespace HeuristicLab.EvolutionTracking {
  public interface IVertex : IItem {
    string Id { get; }
    IEnumerable<IArc> InArcs { get; set; }
    IEnumerable<IArc> OutArcs { get; set; }

    int InDegree { get; }
    int OutDegree { get; }
    int Degree { get; }

    string Label { get; set; }
    double Weight { get; set; }

    void AddForwardArc(IVertex target, double weight = 0.0, object content = null);
    void AddForwardArc(IArc arc);
    void AddReverseArc(IVertex target, double weight = 0.0, object content = null);
    void AddReverseArc(IArc arc);

    object Content { get; set; }
  }

  public interface IVertex<T> : IVertex
  where T : class {
    new T Content { get; set; }
  }
}
